<?php

namespace Blackbirdchess\Service\Import\Readers;

use Blackbirdchess\Service\Results\Enums\Color;
use Blackbirdchess\Service\Results\Enums\Gender;
use Blackbirdchess\Service\Results\Enums\Result;
use Blackbirdchess\Service\Results\Enums\Title;
use Blackbirdchess\Service\Results\Pairing;
use Blackbirdchess\Service\Results\Player;
use Blackbirdchess\Service\Results\Round;
use Blackbirdchess\Service\Results\Tournament;

class Trf16 implements \Blackbirdchess\Service\Import\Interfaces\ReaderInterface
{

    /**
     * The tournament
     *
     * @var Tournament
     */
    public $Tournament;

    private $playerIndexes;

    /**
     * @inheritDoc
     */
    public function read(string $filename): void
    {
        $file = explode("\n", str_replace("\r", "", file_get_contents($filename)));
        $this->Tournament = new Tournament();
        $this->playerIndexes['0000'] = NULL;
        $this->playerIndexes['    '] = NULL;
        foreach ($file as $line) {
            $datatype = substr($line, 0, 3);

            switch($datatype) {
                case '012':
                    $this->Tournament->Name = substr($line, 4);
                    break;
                case '022':
                    $this->Tournament->OrganiserPlace = substr($line, 4);
                    break;
                case '032':
                    $this->Tournament->OrganiserCountry = substr($line, 4);
                    break;
                case '042':
                    $this->Tournament->StartDate = \DateTime::createFromFormat("Y/m/d", substr($line, 4));
                    break;
                case '052':
                    $this->Tournament->EndDate = \DateTime::createFromFormat("Y/m/d", substr($line, 4));
                    break;
                case '102':
                case '112':
                    $this->Tournament->addArbiter(substr($line, 4));
                    break;
                case '001':
                    $this->readPlayerData($line);
                    break;
            }
        }

        $this->setOpponentIndexToPlayerObj();
        $this->Tournament->pairingsToRounds();
    }

    private function readPlayerData(string $line): void
    {
        $player = new Player();
        $player->InitialRank = substr($line, 4, 4);
        switch (substr($line, 9, 1))
        {
            case 'm':
                $player->Gender = Gender::MALE;
                break;
            case 'w':
                $player->Gender = Gender::FEMALE;
                break;
            default:
                $player->Gender = NULL;
                break;
        }

        switch (trim(substr($line, 10, 3)))
        {
            case 'GM':
                $player->Title = Title::GM;
                break;
            case 'IM':
                $player->Title = Title::IM;
                break;
            case 'WGM':
                $player->Title = Title::WGM;
                break;
            case 'FM':
                $player->Title = Title::FM;
                break;
            case 'WIM':
                $player->Title = Title::WIM;
                break;
            case 'CM':
                $player->Title = Title::CM;
                break;
            case 'WFM':
                $player->Title = Title::WFM;
                break;
            case 'WCM':
                $player->Title = Title::WCM;
                break;
            default:
                $player->Title = NULL;
                break;
        }

        $player->Name = trim(substr($line, 14, 32));
        $player->setElo('Fide', (int)trim(substr($line, 48, 4)));
        $player->Nation = trim(substr($line, 53, 3));
        $player->setId('Fide', (int)trim(substr($line, 57, 10)));
        $player->DateOfBirth = \DateTime::createFromFormat("Y/m/d", substr($line, 69, 10));
        $player->Points = (float)trim(substr($line, 80, 4));
        $player->Rank = (float)trim(substr($line, 85, 3));

        $pairingData = substr($line, 91);

        //read pairing round 1
        $pairing = new Pairing();
        $round = new Round();
        $round->RoundNo = 1;
        $round->Date = \DateTime::createFromFormat("y/m/d", '23/01/14');
        $this->Tournament->addRound($round);
        $pairing->Round = 1;
        $pairing->Player = $player;
        $pairing->OpponentIndex = substr($line, 91, 4);

        switch (trim(substr($line, 96, 1)))
        {
            case 'w':
                $pairing->Color = Color::WHITE;
                break;
            case 'b':
                $pairing->Color = Color::BLACK;
                break;
            case '-':
            default:
                $pairing->Color = Color::NONE;
                break;
        }

        switch (strtoupper(trim(substr($line, 98, 1))))
        {
            case '-':
            case 'Z':
                $pairing->Result = Result::ABSENT;
                break;
            case '+':
                $pairing->Result = Result::WON_FORFAIT;
                break;
            case 'W':
            case '1':
                $pairing->Result = Result::WON;
                break;
            case 'D':
            case '=':
                $pairing->Result = Result::DRAW;
                break;
            case 'L':
            case '0':
                $pairing->Result = Result::LOST;
                break;
            case 'H':
            case 'F':
            case 'U':
                $pairing->Result = Result::WON_BYE;
                break;
            default:
                $pairing->Color = Result::NONE;
                break;
        }
        $this->Tournament->addPairing($pairing);
        $this->playerIndexes[$player->InitialRank] = $player;
        $this->Tournament->addPlayer($player);
    }

    private function setOpponentIndexToPlayerObj()
    {
        foreach($this->Tournament->Pairings as &$pairing) {
            $pairing->Opponent = $this->playerIndexes[$pairing->OpponentIndex];
        }
    }
}
